package de.hdm_stuttgart.mi.sda1;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.*;
import org.jdom2.input.SAXBuilder;

import java.io.File;
import java.io.IOException;
import java.sql.*;

public class Company2Rdbms implements AutoCloseable {

  static private final Logger log = LogManager.getLogger(Company2Rdbms.class);

  static private final String
          insertLectureSql = "INSERT INTO Department VALUES (?,  ?)",
      insertMemberSql = "INSERT INTO Member VALUES (?, ?, ?)";

  private Connection conn;

  private Element company;

  private PreparedStatement insertDepartment, insertMember;

  public Company2Rdbms() {
    try {
      conn = DriverManager.getConnection(
          "jdbc:postgresql://localhost/hdm", "hdmuser", "XYZ");
      insertDepartment = conn.prepareStatement(insertLectureSql);
      insertMember = conn.prepareStatement(insertMemberSql);
    } catch (final SQLException e) {
      log.error("Unable to establish connection, exiting", e);
      System.exit(1);
    }

    final SAXBuilder builder = new SAXBuilder();
    try {
      company = builder.build(new File("Schema/sampledata.xml")).getRootElement();
    } catch (final JDOMException e) {
      log.error("Unable to parse input, exiting", e);
      System.exit(1);
    } catch (final IOException e) {
      log.error("Unable to read input, exiting", e);
      System.exit(1);
    }
  }

  public void execute() {
    log.info("Parsing XML input file");
    company.getChild("departments").getChildren("department").forEach(this::handleTopLevelDepartment);
  }

  private void handleTopLevelDepartment(final Element department) {
    handleDepartment(department, null);
  }

  private void handleDepartment(final Element department, final String parent) {
    final String name = department.getChild("name").getValue();
    log.info("Try inserting department '" + name + "'");
    try {
      insertDepartment.setString(1, name);
      insertDepartment.setString(2, parent);
      insertDepartment.execute();
    } catch (final SQLIntegrityConstraintViolationException e) {
      log.info("Possibly existing department");
    } catch (final SQLException e) {
      log.error("Failed to insert department 'name = '" + name + "': ", e);
    }
    department.getChildren("member").forEach(m -> handleMember(m, name));
    department.getChildren("department").forEach(d ->handleDepartment(d, name));
  }

  private void handleMember(final Element member, final String departmentName) {
    final String nickname = member.getAttributeValue("nickname"),
            memberName =member.getValue();

    log.info("Try inserting member '" + nickname + "'");
    try {
      insertMember.setString(1, nickname);
      insertMember.setString(2, memberName);
      insertMember.setString(3, departmentName);
      insertMember.execute();
    } catch (final SQLIntegrityConstraintViolationException e) {
      log.info("Possibly existing member");
    } catch (final SQLException e) {
      log.error("Failed to insert member 'nickname = '" + nickname + "': ", e);
    }
  }


  @Override
  public void close() {
    try {
      conn.close();
    } catch (final SQLException e) {
      e.printStackTrace();
    }
  }
}